@@ -1,5 +1,14 @@
 # FIXME
-fixme = '''
+from devapp.tools import tabulate, termwidth, action_flags, define_flags
+from absl import app, flags
+from xml.etree import ElementTree as ET
+from inspect import signature
+import textwrap
+import sys
+import os
+import io
+
+fixme = """
 Remove all this.
 If colors are important, use the xml output of --helpfull.
 [2020-10-07 22:16] Done. Still due though:
@@ -8,20 +17,10 @@ in sync wiht absl cli flags.
 (do_action of old devapps)
 
 Remove the bloat sooner or later...
-'''
+"""
 
 # TODO: Display use cases and detailled help attributes (see devapp/app.py flags)
 
-import io
-import os
-import sys
-import textwrap
-from inspect import signature
-from xml.etree import ElementTree as ET
-
-from absl import app, flags
-from devapp.tools import tabulate, termwidth, action_flags, define_flags
-
 
 def func_from_partial(f):
     while not hasattr(f, '__module__'):
@@ -32,7 +31,7 @@ def func_from_partial(f):
 
 
 def call_doc(obj, level=2, render=False):
-    """ Used by devapps.app.py for -hh and -hhh"""
+    """Used by devapps.app.py for -hh and -hhh"""
     raise Exception('call_doc not implemented')
     f, kw = func_from_partial(obj)
     if type(f) == type:
@@ -54,7 +53,8 @@ def call_doc(obj, level=2, render=False):
     return out
 
 
-deindent = lambda d: ('\n'.join([l.strip() for l in d.splitlines()]).strip()) + '\n'
+def deindent(d):
+    return ('\n'.join([l.strip() for l in d.splitlines()]).strip()) + '\n'
 
 
 def func_doc(obj, level=1):
@@ -99,7 +99,11 @@ def class_doc(cls, level=1, hir=0, out=None):
 
         elif callable(c):
             funcs.append(
-                {'short': '', 'long': k, 'expl': '\n'.join(func_doc(c, level=level)),}
+                {
+                    'short': '',
+                    'long': k,
+                    'expl': '\n'.join(func_doc(c, level=level)),
+                }
             )
     # FIXME
     add_md_list(funcs, out)
@@ -135,7 +139,8 @@ def term_widths(have_match):
 
 
 def term_line(line_spec, widths, have_match):
-    """One line of output - have_match typically [main module name] """
+    """One line of output - have_match typically [main module name]"""
+    # if 'cache' in str(line_spec) or 'filtered' in str(line_spec): breakpoint()   # FIXME BREAKPOINT
     m = line_spec
     is_action = m.get('action')
     # if is_action:
@@ -155,9 +160,13 @@ def term_line(line_spec, widths, have_match):
         if colmn == 'meaning':
             choi = m.get('choices')
             if choi:
-                choi = ' [%s]' % ','.join(choi)
-                if have_match or len(v) + len(choi) <= w:
-                    v += col('choices') + choi
+                v += m['opts']
+
+                # breakpoint()   # FIXME BREAKPOINT
+                # choi = ' [%s]' % ','.join(choi)
+                # if have_match or len(v) + len(choi) <= w:
+                #     v += col('choices') + choi
+
             det = m.get('details')
             if det:
                 # det = ' 🗎 ' + det
@@ -212,7 +221,10 @@ def to_terminal(flags, widths, match=None):
 
 
 def parse_xml_help(xml_help, match, cli_actions=None):
-    """parse --helpxml output of absl"""
+    """parse --helpxml output of absl
+    cli_actions: the action possibly given to the cli
+
+    """
 
     class types:
         def do_float(c, el):
@@ -223,14 +235,19 @@ def parse_xml_help(xml_help, match, cli_actions=None):
 
         def do_string(c, el):
             m = c['meaning']
-            c['meaning'] = m  #'\x1b[1;38;5;124m%(meaning)s\x1b[0;m' % c
+            c['meaning'] = m  # '\x1b1;38;5;124m%(meaning)s\x1b0;m' % c
 
         def do_comma_separated_list_of_strings(c, el):
             pass
 
         def do_string_enum(c, el):
             c['choices'] = [e.text for e in el]
-            c['meaning'] = c['meaning'].split('>: ', 1)[-1]
+            opts = '|'.join([e.text for e in el if e.tag == 'enum_value'])
+            c['opts'] = f'<{opts}>'
+            m = c['meaning']
+            if '>:' in m:   # split off the opts if given in meaning, we have them in opts
+                m = m.split('>:', 1)[1]
+            c['meaning'] = m
 
         def do_multi_string_enum(c, el):
             return types.do_string_enum(c, el)
@@ -240,7 +257,7 @@ def parse_xml_help(xml_help, match, cli_actions=None):
 
     tree = ET.fromstring(xml_help)
     m, items = {}, {}
-    r = {'program': tree[0].text, 'usage': tree[0].text, 'flags': m}
+    j = {'program': tree[0].text, 'usage': tree[0].text, 'flags': m, 'actions': []}
     for fs in tree[2:]:
         f = fs
         c = {}
@@ -264,7 +281,9 @@ def parse_xml_help(xml_help, match, cli_actions=None):
         items[c['name']] = {'pos': len(l) - 1, 'mod': c['file']}
 
     have = set()
+    # action_flags registered by flag tools at flag class parsing
     for f, af in action_flags.items():
+        # if f == 'droplet_list': breakpoint()
         k = af['key']
         if k in have:
             continue
@@ -273,37 +292,33 @@ def parse_xml_help(xml_help, match, cli_actions=None):
         if not item:
             continue  # not match
         p, mod = item['pos'] + 1, item['mod']
-        m[mod][p - 1]['action'] = True
-        m[mod][p - 1]['is_default'] = af['is_default']
+        modflags = m[mod]
+        modflags[p - 1]['action'] = True
+        modflags[p - 1]['is_default'] = af['is_default']
         # afp.append([p - 1, mod, k, True])
-        if af in cli_actions:
-            f = m[mod][p]
-            while f['name'].startswith(k + '_'):
-                f['action'] = True
-                f['action_flag'] = True
+        # we must now, when an action is given on CLI, list also it's action flags,
+        # defined within this help show module. They'll start with the action + _:
+        if af in cli_actions and p < len(modflags):
+            j['actions'].append(modflags[p - 1])
+            f = modflags[p]
+            fn = f['name']
+            # mind e.g. droplet_list action and droplet_list_no_cache action:
+            while fn.startswith(k + '_') and not fn in action_flags:
+                j['actions'].append(f)
+                f['action'] = f['action_flag'] = True
+                f['action_name'] = k
                 f['name'] = f['name'].split(k + '_', 1)[1]
+                # if f['name'] == 'private_network': breakpoint()   # FIXME BREAKPOINT
+                if p > len(modflags):
+                    break
                 p += 1
-    # if afp:
-    #     fs = r['flags']
-    #     l = []
-    #     m = {}
-    #     for a in afp:
-    #         mod = a[1]
-    #         action = a[2]
-    #         main = a[3]
-    #         f = fs[mod].pop(a[0] + m.get(mod, 0))
-    #         m[mod] = m.setdefault(mod, 0) - 1
-    #         l.append(f)
-    #         if main:
-    #             f['action'] = True
-    #             f['default'] = 'ACTION'
-    #         else:
-    #             f['name'] = f['name'].split(action + '_', 1)[1]
-    #     fs['actions'] = l
-    return r
+                f = modflags[p]
+                fn = f['name']
+
+    return j
 
 
-invisible_sep = u'\u2063'
+invisible_sep = '\u2063'
 
 
 def get_argv_val(k, ign_val=''):
@@ -320,30 +335,34 @@ def get_argv_val(k, ign_val=''):
 
 
 # coloring themeable,i.e. only using the base colors which themes modify:
-ansi_col = lambda nr: '\033[%sm' % nr
-col = lambda c: ansi_col(
-    {
-        'name': '1;33',
-        'default': '0;31',
-        'meaning': '0;38;5;245',
-        'choices': '0;38',
-        'details': '0;38;5;241',
-        'short_name': '0;32',
-        'action': '0;33',
-        'default_action': '1;33',
-    }.get(c)
-)
+def ansi_col(nr):
+    return '\033[%sm' % nr
+
+
+def col(c):
+    return ansi_col(
+        {
+            'name': '1;33',
+            'default': '0;31',
+            'meaning': '0;38;5;245',
+            'choices': '0;38',
+            'details': '0;38;5;241',
+            'short_name': '0;32',
+            'action': '0;33',
+            'default_action': '1;33',
+        }.get(c)
+    )
 
 
 def color_usage(*a, main_module, full=None, **kw):
     """
-        colorizes flag keys and match from --help <match>
+    colorizes flag keys and match from --help <match>
     """
     ret = []
     d = main_module.__doc__
     if d:
         ret.append('\n' + d.strip() + '\n\n')
-    add = lambda s, end='\n': ret.append(s + end)
+    add = lambda s, end='\n', ret=ret: ret.append(s + end)
     # catch the original output:
     hof = get_argv_val('--help_output_fmt')
     hof = hof if hof else 'terminal'
@@ -389,12 +408,13 @@ def color_usage(*a, main_module, full=None, **kw):
     if hof != 'terminal':
         r = tabulate()
 
-    def do(fn_mod):
+    w = term_widths(match)
+
+    def do(fn_mod, w=w, flgs=None):
         # module headline:
-        flgs = all_flgs[fn_mod]
+        flgs = all_flgs[fn_mod] if flgs is None else flgs
         if len(all_flgs) > 1:
             add(ansi_col('1;34') + fn_mod)
-        w = term_widths(match)
         add(to_terminal(flgs, w, match=match))
 
     # main_help = all_flgs.pop(main_module.__name__, [])
@@ -402,10 +422,13 @@ def color_usage(*a, main_module, full=None, **kw):
     n = main_module.__name__
     if full:
         [do(fn_mod) for fn_mod in sorted(all_flgs) if not fn_mod in (n, 'actions')]
+
     if n in all_flgs:
         do(n)
-    if all_flgs.get('actions'):
-        do('actions')
+    # when user has selected an action on CLI, show it last, again:
+    if j.get('actions'):
+        add(ansi_col('1;34') + '\nSelected Action')
+        do('actions', flgs=j['actions'])
 
     # show main module's flags last - always:
     add('\033[0m')
